require 'yaml'

require 'custom_attributes/acts_as/acts_as_customizable'
require 'custom_attributes/acts_as/acts_as_custom_field'
require 'custom_attributes/acts_as/acts_as_custom_value'

require 'custom_attributes/field_type'
require 'custom_attributes/field_types/unbounded'
require 'custom_attributes/field_types/list'
require 'custom_attributes/field_types/numeric'

require 'custom_attributes/field_types/bool_field_type'
require 'custom_attributes/field_types/date_field_type'
require 'custom_attributes/field_types/float_field_type'
require 'custom_attributes/field_types/text_field_type'
require 'custom_attributes/field_types/string_field_type'
require 'custom_attributes/field_types/list_field_type'
require 'custom_attributes/field_types/int_field_type'

require 'custom_attributes/custom_field_value'

require 'custom_attributes/api/custom_attributes_controller_helper'

require 'custom_attributes/concerns/searchable'

module CustomAttributes
  # Gem configuration credits to: https://stackoverflow.com/questions/6233124/where-to-place-access-config-file-in-gem#10112179
  @config = {
    search_user: ENV["ELASTICSEARCH_USER"] || 'elastic',
    search_pass: ENV["ELASTICSEARCH_PASSWORD"] || 'changeme',
    search_host: ENV["ELASTICSEARCH_HOST"] || 'http://localhost:9200',
    search_index_prefix: ENV["ELASTICSEARCH_INDEX_PREFIX"] || 'es'
  }

  @valid_config_keys = @config.keys

  # Configure through hash
  def self.configure(opts = {})
    opts.each { |k, v| @config[k.to_sym] = v if @valid_config_keys.include? k.to_sym }

    after_configuration
  end

  def self.config
    @config
  end

  def self.after_configuration
    # Set up ElasticSearch
    uri = URI(@config[:search_host])

    if @config[:search_user].present? && @config[:search_pass].present?
      host = "#{uri.scheme}://#{@config[:search_user]}:#{@config[:search_pass]}@#{uri.host}"
    else
      host = "#{uri.scheme}://#{uri.host}"
    end

    host = "#{host}:#{uri.port}" if uri.port.present?

    Elasticsearch::Model.client = Elasticsearch::Client.new host: host
  end
end
